#
# Author: Alberto Granzotto <agranzot@gmail.com>
#
# This file may be distributed and/or modified under the terms of
# the GNU General Public License version 2 as published by
# the Free Software Foundation.
#
# This file is distributed without any warranty; without even the implied
# warranty of merchantability or fitness for a particular purpose.
# See "COPYING" in the source distribution for more information.
#

import re
import os
import sys

class PluginManager:
    def __init__(self, dir='.', base_module='plugins'):
        print 'PluginManager::__init__()'
        sys.path.append(dir[:-1])
        self.dir = dir
        self.base_module = base_module
        self.modules = {}
        self.instances = {}
        self.loaded = []

    def __load_modules(self):
        print 'PluginManager::__load_modules()'
        py = re.compile('(.*)\.py$', re.IGNORECASE)
        self.modules = {}
        for filename in os.listdir(self.dir+self.base_module):
            m = py.match(filename)
            if m:
                module = self.base_module + '.' + m.group(1)
                print 'PluginManager::__load_modules() module:', module
                try:
                    # new python versions
                    self.modules[m.group(1)] = (__import__(module, fromlist=['*']))
                except:
                    try:
                        # old ones
                        self.modules[m.group(1)] = (__import__(module, '', '', 'anything'))
                    except:
                        pass
                    pass
        print 'PluginManager::__load_modules() modules:', self.modules

    def __unload_modules(self):
        print 'PluginManager::__unload_modules()'
        self.modules = {}
        
    def __destroy_instances(self):
        print 'PluginManager::__destroy_instances()'
        for name, instance in self.instances.items():
            instance.stop()
        self.instances = {}

    def __create_instances(self):
        print 'PluginManager::__create_instances()'
        self.__destroy_instances()
        for name, module in self.modules.items():
            try:
                self.instances[name] = module.main_class()
            except:
                pass

    def load(self):
        print 'PluginManager::load()'
        self.__load_modules()
        self.__create_instances()
        
    def unload(self):
        print 'PluginManager::unload()'
        self.__destroy_instances()
        self.__unload_modules()
        
if __name__ == '__main__':
    print 'main()'
    p = PluginManager()

